from typing import Optional
from abstra_internals.execution.execution import (
    UnsetStageRun,
    get_current_execution_throwable,
)


def get_data(key: Optional[str] = None):
    execution = get_current_execution_throwable()
    stage_run = execution.stage_run
    if not stage_run:
        raise UnsetStageRun()
    return stage_run.data.get(key) if key else stage_run.data


def set_data(key, value):
    execution = get_current_execution_throwable()
    stage_run = execution.stage_run
    if not stage_run:
        raise UnsetStageRun()
    stage_run.data[key] = value



from ....repositories.stage_run import StageRun
from ....repositories.project.project import ActionStage


def action_strategy(stage: ActionStage, stage_run: StageRun, transition_type: str):
    if transition_type == "forms:abandoned" and not stage.is_initial:
        return [dict(stage=stage_run.stage, data=stage_run.data)]

    next_stage_ids = [
        t.target_id for t in stage.workflow_transitions if t.type == transition_type
    ]

    return [dict(stage=stage_id, data=stage_run.data) for stage_id in next_stage_ids]



from concurrent import futures

executor = futures.ThreadPoolExecutor()


def threaded(func):
    def wrapper(*args, **kwargs):
        return executor.submit(func, *args, **kwargs)

    return wrapper



from unittest import TestCase
from .iterator import iterator_strategy
from abstra_internals.repositories.stage_run import LocalStageRunRepository
from abstra_internals.repositories.project.project import (
    IteratorStage,
    WorkflowTransition,
)


class IteratorStrategyTest(TestCase):
    def setUp(self) -> None:
        self.repository = LocalStageRunRepository()

    def test_simple_foreach(self):
        stage = IteratorStage(
            id="s1",
            title="title",
            variable_name="foo",
            workflow_position=(0, 0),
            workflow_transitions=[
                WorkflowTransition(
                    id="t1",
                    condition_value=None,
                    target_id="target_id",
                    target_type="target_type",
                    type="iterators:each",
                )
            ],
        )
        stage_run = self.repository.create_initial(stage="s1", data={"foo": [1, 2, 3]})
        result = iterator_strategy(stage, stage_run)
        self.assertEqual(
            result,
            [
                dict(stage="target_id", data=dict(item=1)),
                dict(stage="target_id", data=dict(item=2)),
                dict(stage="target_id", data=dict(item=3)),
            ],
        )

    def test_empty_when_not_list(self):
        stage = IteratorStage(
            id="s1",
            title="title",
            variable_name="foo",
            workflow_position=(0, 0),
            workflow_transitions=[
                WorkflowTransition(
                    id="t1",
                    condition_value=None,
                    target_id="target_id",
                    target_type="target_type",
                    type="iterators:each",
                )
            ],
        )
        stage_run = self.repository.create_initial(stage="s1", data={"foo": 1})
        result = iterator_strategy(stage, stage_run)
        self.assertEqual(result, [])

    def test_empty_when_not_found(self):
        stage = IteratorStage(
            id="s1",
            title="title",
            variable_name="foo",
            workflow_position=(0, 0),
            workflow_transitions=[
                WorkflowTransition(
                    id="t1",
                    condition_value=None,
                    target_id="target_id",
                    target_type="target_type",
                    type="iterators:each",
                )
            ],
        )
        stage_run = self.repository.create_initial(stage="s1", data={})
        result = iterator_strategy(stage, stage_run)
        self.assertEqual(result, [])
